@@ -64,24 +64,25 @@ shinyServer(function(input, output, session) {
   
   data.env <- function() wppExplorer:::wpp.data.env
     
+  year.range <- reactiveValues(min=1955, max=2100)
+    
   output$yearUI <- renderUI({
-	data <- indicatorData()
-  	if(nrow(data)==0) return(NULL)
-  	animationOptions(interval = 2000)
-    yearRange <- range(data$Year)
-    value <- yearRange[1]
-    #print(c('slider1: ', value, yearRange, data.env()$year.range, input$year))
-    if(is.null(data.env()$year.range)) wppExplorer:::set.data.env('year.range', yearRange)
-    else {
-    	if(data.env()$year.range[1] != yearRange[1]) {
-    		wppExplorer:::set.data.env('year.range', yearRange)
-    		if(!is.null(input$year) && input$year >= yearRange[1]) value <- input$year
-    	}
-    }
-    #print(c('slider2: ', value, yearRange, data.env()$year.range, input$year))
+  	animationOptions(interval = 3000)
     sliderInput('year', h5('Year:'), sep="",
     			animate=TRUE,
-                min=yearRange[1], max=yearRange[2], value = value, step=5)
+                min=isolate(year.range$min), max=isolate(year.range$max), value = 2015, step=5)
+  })
+  
+  observe({
+  	# update the year slider if the indicator changes and its data have a different time range
+  	if(is.null(input$year)) return(NULL)
+  	data <- indicatorData()
+  	if(nrow(data)==0) return(NULL)
+  	yearRange <- range(data$Year)
+  	if(year.range$min == yearRange[1]) return(NULL)
+    isolate(year.range$min <- yearRange[1])
+    value <- max(yearRange[1], isolate(input$year))	
+  	updateSliderInput(session, "year", min=yearRange[1], value=value) 	
   })
   
   output$indicatorDesc <- renderText({
@@ -211,6 +212,21 @@ shinyServer(function(input, output, session) {
 	data
   	})
   	
+  output$ghist <- renderGvis({
+  	data <- data()
+  	if(is.null(data) || nrow(data)<=0) return(NULL)
+  	# filter out aggregations
+  	data <- merge(data[,c('charcode', 'value')], data.env()$iso3166[data.env()$iso3166$is.country, c('charcode', 'name'), drop=FALSE], by='charcode')
+  	data <- data[,c('name', 'value')]
+  	xlim <- if(input$fiXscaleHist) rangeForAllYears() else range(data$value, na.rm=TRUE)
+  	#browser()
+  	options <- list(title=paste(input$year), legend="{ position: 'none' }", colors="['green']", 
+  						height="500px", width="900px", histogram=paste0("{bucketSize: ", diff(xlim)/30, "}"))
+  						
+  	options$hAxis <- paste0("{maxAlternation: 1,  minValue:", xlim[1], ", maxValue:", xlim[2], "}")
+  	gvisHistogram(data, options=options)
+  })
+  
   output$hist <- renderPlot({
   	data <- data()
   	if(is.null(data) || nrow(data)<=0) return(NULL)
@@ -373,12 +389,19 @@ shinyServer(function(input, output, session) {
   output$age.profileF <- reactive({
 	show.age.profile('F', indicator.fun(), input$aprofile.logscale, input$year)
   })
+  reset.trend.data <- function() ggplot.data$trends <- NULL
+  reset.pyramid.data <- function() ggplot.data$pyramid <- NULL
+  
+  ggplot.data <- reactiveValues(trends=NULL, pyramid=NULL)
+  trends.ranges <- reactiveValues(year=NULL, value=NULL)
   
   output$probtrends <- renderPlot({
+  	reset.trend.data()
   	data <- get.trends.nocast()
   	if(is.null(data)) return(data)
   	data <- data$casted
-  	low <- get.trends.low()
+  	low <- NULL
+  	if (!input$trend.logscale) low <- get.trends.low()
   	if(!is.null(low)) {
   		high <- get.trends.high()
   		colnames(low$casted) <- sub('value', 'low', colnames(low$casted))
@@ -392,7 +415,17 @@ shinyServer(function(input, output, session) {
   		for (col in grep('low|high', colnames(data)))
   			data[idx,col] <- data$value[idx]
   	}
+  	if(!is.null(trends.ranges$year)) {
+  		data.zoom <- subset(data, Year >= trends.ranges$year[1] & Year <= trends.ranges$year[2] & 
+  								value >= trends.ranges$value[1] & value <= trends.ranges$value[2])
+  		if(nrow(data.zoom) == 0) {
+  			trends.ranges$year <- trends.ranges$value <- NULL
+  		} else data <- data.zoom
+  	}
   	g <- ggplot(data, aes(x=Year,y=value,colour=charcode, fill=charcode)) + geom_line() + theme(legend.title=element_blank())
+  	if (input$trend.logscale) g <- g + coord_trans(y="log2")
+  	isolate(ggplot.data$trends <- data)
+  	
   	if(!is.null(low)) {
   		line.data <- NULL
   		for(i in 3:1) {
@@ -401,7 +434,11 @@ shinyServer(function(input, output, session) {
   			g <- g + geom_ribbon(aes_string(ymin=colnames(data)[idx][1], ymax=colnames(data)[idx][2], 
   											fill="charcode", colour="charcode", linetype=NA), alpha=c(0.3, 0.2, 0.1)[i])
   			if(!is.null(line.data)) colnames(line.data) <- c('charcode', 'Year', 'low', 'high', 'variant')
-  			line.data <- rbind(line.data, setNames(cbind(data[,c(1,2,idx)], wppExplorer:::.get.pi.name.for.label(i)), colnames(line.data)))		
+  			line.data <- rbind(line.data, setNames(cbind(data[,c(1,2,idx)], wppExplorer:::.get.pi.name.for.label(i)), colnames(line.data)))
+  			tmp1 <- tmp2 <- data
+  			tmp1[,'value'] <- data[,colnames(data)[idx][1]]
+  			tmp2[,'value'] <- data[,colnames(data)[idx][2]]
+  			isolate(ggplot.data$trends <- rbind(ggplot.data$trends, tmp1, tmp2))
   		}
   		colnames(line.data) <- c('charcode', 'Year', 'low', 'high', 'variant')
 		
@@ -409,9 +446,27 @@ shinyServer(function(input, output, session) {
   		g <- g + geom_line(data=line.data, aes(y=high, linetype=variant, colour=charcode))
   		g <- g + scale_linetype_manual(values=c("80%"=2, '1/2child'=4, "95%"=3), na.value=0)
   	}
+  	isolate(ggplot.data$trends <- merge(ggplot.data$trends, data.env()$iso3166[,c('charcode', 'name')], by='charcode', sort=FALSE))
   	g
   })
   
+  observeEvent(input$probtrends_zoom, {
+  	trends.ranges$year <- c(round(input$probtrends_zoom$xmin,0), round(input$probtrends_zoom$xmax,0))
+  	trends.ranges$value <- c(input$probtrends_zoom$ymin, input$probtrends_zoom$ymax)
+  })
+
+  observeEvent(input$probtrends_zoom_reset, {
+  	trends.ranges$year <- NULL
+  	trends.ranges$value <- NULL
+  })
+  
+  output$probtrends_selected <- renderText({
+  	if(is.null(input$probtrends_values)) return(" ")
+  	selected <- nearPoints(ggplot.data$trends, input$probtrends_values, maxpoints = 1, threshold=10)
+	if(nrow(selected) == 0) return(" ")
+	paste0("Year = ", selected$Year, ', Value = ', round(selected$value,3), ", Country: ", selected$name)
+	})
+		
   .is.pyramid.indicator <- function() {
   	 if(!indicator.fun() %in% c('tpop', 'tpopF', 'tpopM', 'popagesex')) {
   		df <- data.frame(x=0, y=0, lab='No pyramid data for this indicator.')
@@ -429,7 +484,10 @@ shinyServer(function(input, output, session) {
   	data$tpop <- NULL
   	data
   }
-  .get.pyramid.data <- function(proportion=FALSE) {
+  
+  .get.pyramid.data <- reactive({
+  #.get.pyramid.data <- function(proportion=FALSE) {
+  	proportion <- input$proppyramids
   	data <- pyramid.data()
   	if(proportion) {
   		tpop <- wppExplorer::wpp.indicator('tpop')
@@ -460,33 +518,73 @@ shinyServer(function(input, output, session) {
 	}
   	data <- data[order(data$age.num),]
   	data
-  }
+  })
   
   .print.pyramid <- function(data) {
-  	data.range <- range(data$value, na.rm=TRUE)
-  	g <- ggplot(data, aes(y=value, x=reorder(age, age.num), group=charcode, colour=charcode)) + geom_line(subset=.(sex=='F')) + geom_line(subset=.(sex=='M'), aes(y=-1*value)) + scale_x_discrete(name="") + scale_y_continuous(labels=function(x)abs(x)) + coord_flip() + ggtitle(input$year) + theme(legend.title=element_blank())
+  	data.range <- range(abs(data$value), na.rm=TRUE)
+  	g <- ggplot(data, aes(y=value, x=reorder(age, age.num), group=charcode, colour=charcode)) + 
+  			geom_line(data=subset(data, sex=='F')) + geom_line(data=subset(data, sex=='M')) + 
+  			scale_x_discrete(name="") + scale_y_continuous(labels=function(x)abs(x)) + 
+  			coord_flip() + ggtitle(input$year) + theme(legend.title=element_blank())
   	g <- g + geom_text(data=NULL, y=-data.range[2]/2, x=20, label="Male", colour='black')
   	g <- g + geom_text(data=NULL, y=data.range[2]/2, x=20, label="Female", colour='black')
   	g <- g + geom_hline(yintercept = 0)
   	if(is.element('low', colnames(data))) {
-  		g <- g + geom_ribbon(subset=.(sex=='F'), aes(ymin=low, ymax=high, linetype=NA), alpha=0.3)
-  		g <- g + geom_ribbon(subset=.(sex=='M'), aes(ymin=-high, ymax=-low, linetype=NA), alpha=0.3)
+  		g <- g + geom_ribbon(data=subset(data, sex=='F'), aes(ymin=low, ymax=high, linetype=NA), alpha=0.3)
+  		g <- g + geom_ribbon(data=subset(data, sex=='M'), aes(ymin=high, ymax=low, linetype=NA), alpha=0.3)
   		line.data <- cbind(data, variant=wppExplorer:::.get.pi.name.for.label(3)) # only half-child variant available 
-  		g <- g + geom_line(data=line.data, subset=.(sex=='F'), aes(y=low, linetype=variant, colour=charcode, group=charcode)) # female low
-  		g <- g + geom_line(data=line.data, subset=.(sex=='F'), aes(y=high, linetype=variant, colour=charcode, group=charcode)) # female high
-  		g <- g + geom_line(data=line.data, subset=.(sex=='M'), aes(y=-low, linetype=variant, colour=charcode, group=charcode)) # male low
-  		g <- g + geom_line(data=line.data, subset=.(sex=='M'), aes(y=-high, linetype=variant, colour=charcode, group=charcode)) # male high
+  		g <- g + geom_line(data=subset(line.data, sex=='F'), aes(y=low, linetype=variant, colour=charcode, group=charcode)) # female low
+  		g <- g + geom_line(data=subset(line.data, sex=='F'), aes(y=high, linetype=variant, colour=charcode, group=charcode)) # female high
+  		g <- g + geom_line(data=subset(line.data, sex=='M'), aes(y=low, linetype=variant, colour=charcode, group=charcode)) # male low
+  		g <- g + geom_line(data=subset(line.data, sex=='M'), aes(y=high, linetype=variant, colour=charcode, group=charcode)) # male high
         g <- g + scale_linetype_manual(values=c("80%"=2, '1/2child'=4, "95%"=3), na.value=0)
+        tmp1 <- tmp2 <- data
+		tmp1[,'value'] <- data[,'low']
+  		tmp2[,'value'] <- data[,'high']
+  		isolate(ggplot.data$pyramid <- rbind(ggplot.data$pyramid, tmp1, tmp2))
   	}
   	g
   }
   
+  pyramid.ranges <- reactiveValues(age=NULL, value=NULL)
   output$pyramids <- renderPlot({
   	if(!.is.pyramid.indicator()) return()
-	data <- .get.pyramid.data(proportion=input$proppyramids)
-	.print.pyramid(data)
+  	reset.pyramid.data()
+	#data <- .get.pyramid.data(proportion=input$proppyramids)
+	data <- .get.pyramid.data()
+	male.idx <- which(data$sex=='M')
+	for(col in c('value', 'low', 'high'))
+		if(!is.null(data[[col]])) data[[col]][male.idx] <- -data[[col]][male.idx]
+	if(!is.null(pyramid.ranges$age)) {
+		#print(input$pyramid_zoom)
+  		data.zoom <- subset(data, age.num >= pyramid.ranges$age[1] & age.num <= pyramid.ranges$age[2] & 
+  								value >= pyramid.ranges$value[1] & value <= pyramid.ranges$value[2])
+  		if(nrow(data.zoom) == 0) {
+  			pyramid.ranges$age <- pyramid.ranges$value <- NULL
+  		} else data <- data.zoom
+  	}
+	isolate(ggplot.data$pyramid <- data)
+	g <- .print.pyramid(data)
+	isolate(ggplot.data$pyramid <- merge(ggplot.data$pyramid, data.env()$iso3166[,c('charcode', 'name')], by='charcode', sort=FALSE))
+	g
   })
   
+  observeEvent(input$pyramid_zoom, {
+  	pyramid.ranges$age <- c(round(input$pyramid_zoom$ymin,0), round(input$pyramid_zoom$ymax,0))
+  	pyramid.ranges$value <- c(input$pyramid_zoom$xmin, input$pyramid_zoom$xmax)
+  })
+
+  observeEvent(input$pyramid_zoom_reset, {
+  	pyramid.ranges$age <- NULL
+  	pyramid.ranges$value <- NULL
+  })
+  
+  output$pyramid_selected <- renderText({
+  	if(is.null(input$pyramid_values)) return(" ")
+  	selected <- nearPoints(ggplot.data$pyramid, input$pyramid_values, xvar='value', yvar='age.num', maxpoints = 1, threshold=10)
+	if(nrow(selected) == 0) return(" ")
+	paste0(if(selected$value < 0) "Male " else "Female ", selected$age, ', Value = ', round(abs(selected$value),3), ", Country: ", selected$name)
+	})
   
   # .get.digits <- reactive({
   	# print(wppExplorer:::ind.digits(as.integer(input$indicator)))
